"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Logger = void 0;
const LogLevel_1 = require("./LogLevel");
const Placeholders = require("placeholderz");
const EOL = '\n';
const dateformat = require('dateformat'), objAssign = require('assign-deep');
/**
 * @type {{displayLevels: LogLevel[], messageFormat: string, timeFormat: string, pipe: {[p: string]: "stdout" | "stderr"}}}
 */
let global_parameters = {
    pipe: {
        [LogLevel_1.LogLevel.Debug]: 'stdout',
        [LogLevel_1.LogLevel.Info]: 'stdout',
        [LogLevel_1.LogLevel.Notice]: 'stdout',
        [LogLevel_1.LogLevel.Warning]: 'stdout',
        [LogLevel_1.LogLevel.Error]: 'stdout',
        [LogLevel_1.LogLevel.Critical]: 'stdout'
    },
    displayLevels: Object.keys(LogLevel_1.LogLevel),
    timeFormat: 'dd-mm-yyyy HH:MM:ss.l Z',
    messageFormat: '[ {time} ] [ {fmt_log_level} ] [ {fmt_module} ] {message}'
};
/**
 * @type {{messageTemplate: string}}
 */
const logger_default_parameters = {
    messageTemplate: '{message}',
    muted: false
};
let longest_log_level = Math.max(...Object.values(LogLevel_1.LogLevel).map(v => v.length)), longest_module_name = 0;
class Logger {
    constructor(module_name, parameters) {
        this.module_name = module_name;
        this.params = objAssign({}, logger_default_parameters, parameters);
        if (module_name.length > longest_module_name)
            longest_module_name = module_name.length;
    }
    /**
     * @param {Partial<ILoggerGlobalParameters>} parameters
     */
    static setGlobalParameters(parameters) {
        global_parameters = objAssign({}, global_parameters, parameters);
    }
    /**
     * @returns {ILoggerGlobalParameters}
     */
    static getGlobalParameters() {
        return global_parameters;
    }
    /**
     * @param {LogLevel} log_level
     * @param {string} message
     * @param parameters
     */
    write(log_level, message, ...parameters) {
        if (this.params.muted)
            return;
        if (global_parameters.displayLevels.indexOf(log_level) === -1)
            return;
        const placeholders = {
            time: dateformat(global_parameters.timeFormat),
            log_level: log_level.toString().toUpperCase(),
            fmt_log_level: this.format_log_level(log_level),
            module: this.module_name,
            fmt_module: this.format_module_name(this.module_name),
            message: Placeholders.replaceNamed(this.params.messageTemplate, {
                message: Placeholders.replace(message, parameters)
            })
        };
        const log_message = Placeholders.replaceNamed(global_parameters.messageFormat, placeholders);
        if (typeof (process) !== 'undefined' && process !== null) {
            if (process[global_parameters.pipe[log_level]] !== undefined
                && process[global_parameters.pipe[log_level]].writable) {
                process[global_parameters.pipe[log_level]].write(log_message + EOL);
                return;
            }
            if (process['stdout'] !== undefined && process['stdout'].writable) {
                process.stdout.write(log_message + EOL);
                return;
            }
            // Try to use console.log / console.warn / console.error
        }
        switch (log_level) {
            default:
            case LogLevel_1.LogLevel.Debug:
            case LogLevel_1.LogLevel.Info:
            case LogLevel_1.LogLevel.Notice:
                console.log(log_message);
                break;
            case LogLevel_1.LogLevel.Warning:
                console.warn(log_message);
                break;
            case LogLevel_1.LogLevel.Error:
                console.error(log_message);
                break;
        }
    }
    /**
     * @param {LogLevel} log_level
     * @returns {string}
     * @protected
     */
    format_log_level(log_level) {
        return log_level.toString().toUpperCase().padEnd(longest_log_level, ' ');
    }
    /**
     * @param {string} module_name
     * @returns {string}
     * @protected
     */
    format_module_name(module_name) {
        return module_name.padStart(Math.round(longest_module_name / 2 + module_name.length / 2), ' ')
            .padEnd(longest_module_name, ' ');
    }
    /**
     * @param {string} message
     * @param parameters
     */
    debug(message, ...parameters) {
        this.write(LogLevel_1.LogLevel.Debug, message, ...parameters);
    }
    /**
     * @param {string} message
     * @param parameters
     */
    info(message, ...parameters) {
        this.write(LogLevel_1.LogLevel.Info, message, ...parameters);
    }
    /**
     * @param {string} message
     * @param parameters
     */
    notice(message, ...parameters) {
        this.write(LogLevel_1.LogLevel.Notice, message, ...parameters);
    }
    /**
     * @param {string} message
     * @param parameters
     */
    warn(message, ...parameters) {
        this.write(LogLevel_1.LogLevel.Warning, message, ...parameters);
    }
    /**
     * @param {string} message
     * @param parameters
     */
    warning(message, ...parameters) {
        this.write(LogLevel_1.LogLevel.Warning, message, ...parameters);
    }
    /**
     * @param {string} message
     * @param parameters
     */
    error(message, ...parameters) {
        this.write(LogLevel_1.LogLevel.Error, message, ...parameters);
    }
    /**
     * @param {string} message
     * @param parameters
     */
    critical(message, ...parameters) {
        this.write(LogLevel_1.LogLevel.Critical, message, ...parameters);
    }
}
exports.Logger = Logger;
